// Copyright (c) 2016, Dart Gaming authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'dart:async';

import 'package:collection/collection.dart';
import 'package:meta/meta.dart';

/// Transforms the stream to emit a value determined by [distinct] changes.
///
/// - May specify a custom [equality] to determine what makes a value distinct
///
/// This is similar to the much longer (without three stream transformers):
/// ```dart
/// stream
///   .map(map)
///   .distinct(equality)
///   .map(mapEventAfter);
/// ```
StreamTransformer/*<S, S>*/ observe/*<S, T>*/({
  @required distinct(S input),
  @required T map(S input),
  Equality<T> equality: const DefaultEquality(),
}) {
  T _lastValue;
  return new StreamTransformer/*<S, S>*/ .fromHandlers(
    handleData: (event, sink) {
      final value = distinct(event);
      if (!equality.equals(_lastValue, value)) {
        _lastValue = value;
        sink.add(event);
      }
    },
  );
}
