// Copyright (c) 2016, Dart Gaming authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'package:gamepad/gamepad.dart';
import 'package:gamepad/drivers/virtual.dart';
import 'package:test/test.dart';

void main() {
  group('$Axes', () {
    _SimpleVirtualAxes axes;

    setUp(() => axes = new _SimpleVirtualAxes());

    test('should emit angleDirection when the direction changes', () async {
      axes.triggerUp();
      axes.triggerRight();
      axes.triggerDown();
      axes.triggerLeft();
      axes.triggerLeft();
      axes.close();
      expect(
        await axes.angleDirection.toList(),
        const [
          AxesDirection.up,
          AxesDirection.right,
          AxesDirection.down,
          AxesDirection.left,
        ],
      );
    });

    test('should emit xAxisDirection when the x-axis changes', () async {
      axes.add(const AxesState(angle: 90));
      axes.add(const AxesState(angle: 270));
      axes.add(const AxesState(angle: 185));
      axes.close();
      expect(
        await axes.xAxisDirection.toList(),
        const [
          AxesDirection.right,
          AxesDirection.left,
        ],
      );
    });

    test('should emit yAxisDirection when the y-axis changes', () async {
      axes.add(const AxesState(angle: 45));
      axes.add(const AxesState(angle: 180));
      axes.add(const AxesState(angle: 200));
      axes.close();
      expect(
        await axes.yAxisDirection.toList(),
        const [
          AxesDirection.up,
          AxesDirection.down,
        ],
      );
    });
  });
}

class _SimpleVirtualAxes = VirtualAxes with TriggerDirection;
